#ifndef __GCLAY_H__
#define __GCLAY_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#ifdef WIN32
#define GCLAY_CALL __stdcall
#else
#define GCLAY_CALL
#endif

#pragma pack (push, 1)

// *Forms*

typedef int (GCLAY_CALL *DemandGamingForm_t)();

struct SDemandBrowserForm {
    int ParamSize;
    const char *Url;
    double Width;
    double Height;
	const char *Caption;
    double ClientWidth;
    double ClientHeight;
};

typedef int (GCLAY_CALL *DemandBrowserForm_t)(SDemandBrowserForm *param);

// *Overlay*

typedef int (GCLAY_CALL *GetOverlayState_t)();

    typedef void (GCLAY_CALL *OverlayStateCallback_t)(int overlayState);
// Подписаться на изменение состояния оверлея.
typedef void (GCLAY_CALL *SetOverlayStateCallback_t)(OverlayStateCallback_t callback);

// *Voice*

// Функция вызывается при нажатии юзером хоткея для активации функции push2talk в голосовом чате.
typedef void (GCLAY_CALL *VoiceTalkStart_t)();

// Функция вызывается при нажатии юзером хоткея для отключения функции push2talk в голосовом чате.
typedef void (GCLAY_CALL *VoiceTalkStop_t)();

    typedef void (GCLAY_CALL *VoiceTalkingCallback_t)(const char *persId, int talkingState);
// Подписаться на изменение состояния чата.
typedef void (GCLAY_CALL *VoiceSetTalkingCallback_t)(VoiceTalkingCallback_t callback);

// mode:
// "off"    -	микрофон отключен, звук не передается.
// "ptt"    -	Push-To-Talk, микрофон включается-выключается функциями VoiceTalkStart, VoiceTalkStop
// "auto"   -   Режим активации по голосоу. Микрофон включе всегда, но передача просиходит только в момент когда человек говорит.
//              На начальном этапе: звук идет всегда, но когда человек говорит мы дополнительно показываем логотип динамика
// "always" -	микрофон включен постояно, звук идет постоянно
typedef void (GCLAY_CALL *VoiceTalkSetMode_t)(const char *mode);

// Выключить звук от пользователя persId.
typedef void (GCLAY_CALL *VoiceTalkMute_t)(const char *persId);

// Включить звук от пользователя persId.
typedef void (GCLAY_CALL *VoiceTalkUnmute_t)(const char *persId);

// Запрашивает состояние всех участников звонка для данной игры. Голосовой чат в ответ вызовет функцию VoicePeerStateCallback(persId, peer), для каждого из участников.
// Имеет смысл вызвать эту функцию после добавления игрой колбэка в gclay, что бы передать первоначальную информацию об участниках.
typedef void (GCLAY_CALL *VoiceGetAllPeerState_t)();

// Запрашивает состояние отдельного участника в звонке для данной игры. Голосовой чат в ответ вызовет функцию VoicePeerStateCallback(persId, peer), куда передаст всю информацию.
typedef void (GCLAY_CALL *VoiceGetPeerState_t)(const char *persId);

struct SPeerState {
	int Size;
	const char *PeerId;     // Hash - идентификатор участника. Например "6b7a3b5ee3307bd6df40aacc225eaa2b"
	const char *PersId;     // ID игрока в игре.Строка, хотя на данный момент в строке будет число. Например "1234556"
	int IsConnected;  // Установлено ли соединениес данным участником.
	int CanTalk;      // У участника есть микрофон и он может говорить(бывают такие, у которых только уши).
	int IsTalking;    // Участник говорит в данный момент
	int IsMuted;	  // Участник "заглушен"
};

	// В случае удаления участника из чата, вместо объекта в параметре peer передается null значение.
    typedef void (GCLAY_CALL *VoicePeerStateCallback_t)(const char *persId, SPeerState *peerState);
// Подписаться на изменение состояния участников чата.
typedef void (GCLAY_CALL *VoiceSetPeerStateCallback_t)(VoicePeerStateCallback_t callback);

// Включено ли в самом ИЦ использование голосового чата в играх
typedef int (GCLAY_CALL *GetVoiceTalkEnabled_t)();

#define GC_GCLAY_PATHNAME L"GC_GCLAY_PATHNAME"

#define DEMAND_GAMING_FORM "DemandGamingForm"
#define DEMAND_BROWSER_FORM "DemandBrowserForm"

#define GET_OVERLAY_STATE "GetOverlayState"
#define SET_OVERLAY_STATE_CALLBACK "SetOverlayStateCallback"

#define VOICE_TALK_START "VoiceTalkStart"
#define VOICE_TALK_STOP "VoiceTalkStop"
#define VOICE_SET_TALKING_CALLBACK "VoiceSetTalkingCallback"

#define VOICE_TALK_SET_MODE "VoiceTalkSetMode"
#define VOICE_TALK_MUTE "VoiceTalkMute"
#define VOICE_TALK_UNMUTE "VoiceTalkUnmute"
#define VOICE_GET_ALL_PEER_STATE "VoiceGetAllPeerState"
#define VOICE_GET_PEER_STATE "VoiceGetPeerState"
#define VOICE_SET_PEER_STATE_CALLBACK "VoiceSetPeerStateCallback"
#define GET_VOICE_TALK_ENABLED "GetVoiceTalkEnabled"

#pragma pack (pop)

#ifdef __cplusplus
}
#endif

#endif
