// GameCenterTestApp.cpp 
// Sample application for Game Center game developers

#include "stdafx.h"

#include "GameCenterTestApp.h"

#define USE_GCLAY

#ifdef USE_GCLAY
#include "GCLay.h"
#endif


#define GC_TYPE_ID L"GC_TYPE_ID"
#define GC_PROJECT_ID L"GC_PROJECT_ID"


std::string curlWriteResult;

size_t CURLWriteFunction(void *ptr, size_t size, size_t nmemb, void *stream) {

	size_t sum = nmemb * size;
	if (sum > 0) {
		curlWriteResult.append((const char*)ptr, sum);
	}
	
	return sum;
}

INT_PTR CALLBACK    ProcessMainDialog(HWND, UINT, WPARAM, LPARAM);

// define global variables
LPWSTR gCmdLine;
bool gAuthChecked = false;

int APIENTRY wWinMain(_In_ HINSTANCE hInstance,
                     _In_opt_ HINSTANCE hPrevInstance,
                     _In_ LPWSTR    lpCmdLine,
                     _In_ int       nCmdShow) {
    UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(nCmdShow);

	// Store command line parameters to global variable
	gCmdLine = lpCmdLine;

	curl_global_init(CURL_GLOBAL_ALL);

	// Run main dialog
	DialogBoxW(hInstance, MAKEINTRESOURCE(IDD_MAIN_DIALOG), nullptr, (DLGPROC)ProcessMainDialog);

	curl_global_cleanup();
    return 0;
}

// Get a environment variable
std::wstring GetEnvVarW(const wchar_t *key) {
	std::wstring s;
	s.resize(1024);
	DWORD l = GetEnvironmentVariableW(key, (LPWSTR)s.data(), (DWORD)s.size());
	s.resize((std::size_t)l);
	return s;
}
#ifdef USE_GCLAY

HMODULE gcLayHandle = nullptr;

DemandGamingForm_t demandGamingForm = nullptr;
DemandBrowserForm_t demandBrowserForm = nullptr;

GetOverlayState_t getOverlayState = nullptr;
SetOverlayStateCallback_t setOverlayStateCallback = nullptr;

VoiceSetTalkingCallback_t voiceSetTalkingCallback = nullptr;
VoiceTalkStart_t voiceTalkStart = nullptr;
VoiceTalkStop_t voiceTalkStop = nullptr;
VoiceTalkSetMode_t voiceTalkSetMode = nullptr;
VoiceTalkMute_t voiceTalkMute = nullptr;
VoiceTalkUnmute_t voiceTalkUnmute = nullptr;
VoiceGetAllPeerState_t voiceGetAllPeerState = nullptr;
VoiceGetPeerState_t voiceGetPeerState = nullptr;
VoiceSetPeerStateCallback_t voiceSetPeerStateCallback = nullptr;
GetVoiceTalkEnabled_t getVoiceTalkEnabled = nullptr;

void GCLAY_CALL OverlayStateCallback(int overlayState) {
	// implementation
}

void GCLAY_CALL VoiceTalkingCallback(const char *persId, int talkingState) {
	// implementation
}

void GCLAY_CALL VoicePeerStateCallback(const char *persId, SPeerState *peerState) {
	// implementation
}

void SetupGCLay() {
	if (gcLayHandle == nullptr) {
		std::wstring sGCLayPathname = GetEnvVarW(GC_GCLAY_PATHNAME); // get full GCLay dll filename
		gcLayHandle = GetModuleHandleW(sGCLayPathname.c_str()); // get GCLay module handle

		if (gcLayHandle != nullptr) {
			demandGamingForm = (DemandGamingForm_t)GetProcAddress(gcLayHandle, DEMAND_GAMING_FORM);
			demandBrowserForm = (DemandBrowserForm_t)GetProcAddress(gcLayHandle, DEMAND_BROWSER_FORM);

			getOverlayState = (GetOverlayState_t)GetProcAddress(gcLayHandle, GET_OVERLAY_STATE);
			setOverlayStateCallback = (SetOverlayStateCallback_t)GetProcAddress(gcLayHandle, SET_OVERLAY_STATE_CALLBACK);

			voiceSetTalkingCallback = (VoiceSetTalkingCallback_t)GetProcAddress(gcLayHandle, VOICE_SET_TALKING_CALLBACK);
			voiceTalkStart = (VoiceTalkStart_t)GetProcAddress(gcLayHandle, VOICE_TALK_START);
			voiceTalkStop = (VoiceTalkStop_t)GetProcAddress(gcLayHandle, VOICE_TALK_STOP);
			voiceTalkMute = (VoiceTalkMute_t)GetProcAddress(gcLayHandle, VOICE_TALK_MUTE);
			voiceTalkUnmute = (VoiceTalkUnmute_t)GetProcAddress(gcLayHandle, VOICE_TALK_UNMUTE);
			voiceGetAllPeerState = (VoiceGetAllPeerState_t)GetProcAddress(gcLayHandle, VOICE_GET_ALL_PEER_STATE);
			voiceGetPeerState = (VoiceGetPeerState_t)GetProcAddress(gcLayHandle, VOICE_GET_PEER_STATE);
			voiceSetPeerStateCallback = (VoiceSetPeerStateCallback_t)GetProcAddress(gcLayHandle, VOICE_SET_PEER_STATE_CALLBACK);
			getVoiceTalkEnabled = (GetVoiceTalkEnabled_t)GetProcAddress(gcLayHandle, GET_VOICE_TALK_ENABLED);
		}
	}


	if (setOverlayStateCallback != nullptr) {
		setOverlayStateCallback(OverlayStateCallback);
	}

	if (voiceSetTalkingCallback != nullptr) {
		voiceSetTalkingCallback(VoiceTalkingCallback);
	}

	if (voiceSetPeerStateCallback != nullptr) {
		voiceSetPeerStateCallback(VoicePeerStateCallback);
	}
}

bool DemandGamingForm() {
	if (demandGamingForm != nullptr) {
		return demandGamingForm() != 0;
	}
	return false;
}

bool DemandBrowserForm(const std::string &url, double width, double height, const std::string &caption) {
	if (demandBrowserForm != nullptr) {
		SDemandBrowserForm param;
		param.ParamSize = sizeof(param);
		param.Url = url.c_str();
		param.Width = width;
		param.Height = height;
		param.Caption = caption.c_str();

		return demandBrowserForm(&param) != 0;
	}
	return false;
}

void VoiceTalkStart() {
	if (voiceTalkStart != nullptr) {
		voiceTalkStart();
	}
}

void VoiceTalkStop() {
	if (voiceTalkStop != nullptr) {
		voiceTalkStop();
	}
}

void VoiceTalkMute(const std::string &persId) {
	if (voiceTalkMute != nullptr) {
		voiceTalkMute(persId.c_str());
	}
}

void VoiceTalkUnmute(const std::string &persId) {
	if (voiceTalkUnmute != nullptr) {
		voiceTalkUnmute(persId.c_str());
	}
}

void VoiceGetAllPeerState() {
	if (voiceGetAllPeerState != nullptr) {
		voiceGetAllPeerState();
	}
}

void VoiceGetPeerState(const std::string &persId) {
	if (voiceGetPeerState != nullptr) {
		voiceGetPeerState(persId.c_str());
	}
}

void GetVoiceTalkEnabled() {
	if (getVoiceTalkEnabled != nullptr) {
		getVoiceTalkEnabled();
	}
}

#endif

bool ExtractArgValue(const std::wstring &prefix, const std::wstring &argStr, std::wstring &value) {
	if (argStr.compare(0, prefix.size(), prefix) == 0) {
		value = argStr.substr(prefix.size());
		return true;
	}
	return false;
}

std::wstring GetDlgItemStringW(HWND window, int itemId) {
	std::wstring value;
	value.resize(1024);
	UINT size = GetDlgItemTextW(window, itemId, (LPWSTR)value.data(), (int)value.size());
	value.resize((std::size_t)size);
	return value;
}

std::string UTF8Encode(const std::wstring &ws) {
	std::string s;
	if (!ws.empty()) {
		int sizeNeeded = WideCharToMultiByte(CP_UTF8, 0, ws.data(), (int)ws.size(), nullptr, 0, nullptr, nullptr);
		s.resize((std::size_t)sizeNeeded);
		sizeNeeded = WideCharToMultiByte(CP_UTF8, 0, ws.data(), (int)ws.size(), (LPSTR)s.data(), sizeNeeded, nullptr, nullptr);
		s.resize((std::size_t)sizeNeeded);
	}
	return s;
}

std::wstring UTF8Decode(const std::string &s) {
	std::wstring ws;
	if (!s.empty()) {
		int sizeNeeded = MultiByteToWideChar(CP_UTF8, 0, s.data(), (int)s.size(), nullptr, 0);
		ws.resize((std::size_t)sizeNeeded);
		sizeNeeded = MultiByteToWideChar(CP_UTF8, 0, s.data(), (int)s.size(), (LPWSTR)ws.data(), sizeNeeded);
		ws.resize((std::size_t)sizeNeeded);
	}
	return ws;
}

INT_PTR CALLBACK ProcessMainDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
    UNREFERENCED_PARAMETER(lParam);
	switch (message) {
		case WM_INITDIALOG: {
			{
				// Get exe file name and append command line parameters
				std::wstring sFileName;
				sFileName.resize(MAX_PATH);
				DWORD len = GetModuleFileNameW(0, (LPWSTR)sFileName.data(), (DWORD)sFileName.size());
				sFileName.resize((size_t)len);
				std::wstring str = sFileName;
				str.append(L" ").append(gCmdLine);
				// Output to form	
				SetDlgItemTextW(hDlg, IDC_COMMAND_LINE_EDIT, (LPCWSTR)str.data());
			}
			{
				LPWSTR *argsList;
				int argsCount;
				argsList = CommandLineToArgvW(gCmdLine, &argsCount);
				if (argsList != nullptr) {
					for (LPWSTR *arg = argsList; argsCount > 0; argsCount--, arg++) {
						if ((arg == nullptr) || (*arg == nullptr)) {
							continue;
						}
						std::wstring argStr(*arg);
						std::wstring value;
						if (ExtractArgValue(L"--sz_pers_id=", argStr, value)) {
							SetDlgItemTextW(hDlg, IDC_AUTH_PERSID_EDIT, (LPCWSTR)value.data());
							continue;
						}
						if (ExtractArgValue(L"--sz_token=", argStr, value)) {
							SetDlgItemTextW(hDlg, IDC_AUTH_TOKEN_EDIT, (LPCWSTR)value.data());
							continue;
						}
					}
					LocalFree(argsList);
				}
			}
			{
				// Get Game Center's type ID and project ID of the current game
				std::wstring gcTypeId = GetEnvVarW(GC_TYPE_ID);
				std::wstring gcProjectId = GetEnvVarW(GC_PROJECT_ID);
				// Create a Game Center's current game ID
				std::wstring gcGameId = gcTypeId + L"." + gcProjectId;

				// Output to form	
				std::wstring str;
				str = GC_TYPE_ID;
				str.append(L" = ").append(gcTypeId);
				SendDlgItemMessageW(hDlg, IDC_ENV_VARS_LIST, LB_ADDSTRING, 0, (LPARAM)str.c_str());

				str = GC_PROJECT_ID;
				str.append(L" = ").append(gcProjectId);
				SendDlgItemMessageW(hDlg, IDC_ENV_VARS_LIST, LB_ADDSTRING, 0, (LPARAM)str.c_str());

				str = L"http://admin.dl.mail.ru/gctestauth.php";
				SetDlgItemTextW(hDlg, IDC_AUTH_URL_EDIT, (LPCWSTR)str.data());


	#ifdef USE_GCLAY
				std::wstring sGCLayPathname = GetEnvVarW(GC_GCLAY_PATHNAME);
				str = GC_GCLAY_PATHNAME;
				str.append(L" = ").append(sGCLayPathname);
				SetupGCLay();
				if (gcLayHandle != nullptr) {
					str.append(L" (loaded)");
				}

				SendDlgItemMessageW(hDlg, IDC_ENV_VARS_LIST, LB_ADDSTRING, 0, (LPARAM)str.c_str());
	#endif

				SetDlgItemTextW(hDlg, IDC_GAMEID_EDIT, (LPCWSTR)gcGameId.data());
			}

			return (INT_PTR)TRUE;
		}
		break;
		case WM_COMMAND: {
			if (LOWORD(wParam) == IDC_AUTH_CHECK_BUTTON) {
				if (gAuthChecked) {
					MessageBoxW(hDlg, L"Auth already checked. Auth token can be used only ONCE!", L"Warning", MB_OK | MB_ICONWARNING);
				}
				else {
					gAuthChecked = true;
					CURLcode curlRes;
					CURL *curlHandle;
					curl_httppost *formPost = nullptr;
					curl_httppost *lastPtr = nullptr;
					std::string authPersId = UTF8Encode(GetDlgItemStringW(hDlg, IDC_AUTH_PERSID_EDIT));
					std::string authToken = UTF8Encode(GetDlgItemStringW(hDlg, IDC_AUTH_TOKEN_EDIT));
					std::string authUrl = UTF8Encode(GetDlgItemStringW(hDlg, IDC_AUTH_URL_EDIT));
					curlWriteResult.clear();
					
					curlHandle = curl_easy_init();
					if (curlHandle) {
						curl_formadd(&formPost,
							&lastPtr,
							CURLFORM_COPYNAME, "persId",
							CURLFORM_COPYCONTENTS, authPersId.c_str(),
							CURLFORM_END);

						curl_formadd(&formPost,
							&lastPtr,
							CURLFORM_COPYNAME, "token",
							CURLFORM_COPYCONTENTS, authToken.c_str(),
							CURLFORM_END);

						// curl_easy_setopt(curlHandle, CURLOPT_PROXY, "127.0.0.1:8888");

						curl_easy_setopt(curlHandle, CURLOPT_WRITEFUNCTION, CURLWriteFunction);
						curl_easy_setopt(curlHandle, CURLOPT_URL, authUrl.c_str());
						curl_easy_setopt(curlHandle, CURLOPT_HTTPPOST, formPost);

						curlRes = curl_easy_perform(curlHandle);

						if (curlRes != CURLE_OK) {
							std::wstring errorStr;
							errorStr = L"Network error";

							const char *curlErr = curl_easy_strerror(curlRes);
							if (curlErr != nullptr) {
								errorStr.append(L":\r\n");
								std::string curlErrStr(curlErr);
								errorStr.append(UTF8Decode(curlErrStr));
							} else {
								errorStr.append(L".");
							}
							
							MessageBoxW(hDlg, errorStr.c_str(), L"Error", MB_OK | MB_ICONERROR);
						} else {
							if (curlWriteResult.compare(0, 2, "OK") == 0) {
								MessageBoxW(hDlg, L"Auth OK", L"Info", MB_OK | MB_ICONINFORMATION);
							} else {
								std::wstring errorStr;
								errorStr = L"Auth Error";
								if (!curlWriteResult.empty()) {
									errorStr.append(L":\r\n");
									errorStr.append(UTF8Decode(curlWriteResult));
								}
								MessageBoxW(hDlg, errorStr.c_str(), L"Error", MB_OK | MB_ICONERROR);
							}
						}

						curl_easy_cleanup(curlHandle);

						curl_formfree(formPost);
					}


				}
				return (INT_PTR)TRUE;
			}
			if ((LOWORD(wParam) == IDCLOSE) || (LOWORD(wParam) == IDCANCEL) || (LOWORD(wParam) == IDC_CLOSE_BUTTON)) {
				EndDialog(hDlg, LOWORD(wParam));
				return (INT_PTR)TRUE;
			}
		}
		break;
    }
    return (INT_PTR)FALSE;
}
